/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.PluginProfileImportEditor");

dojo.require("dijit._WidgetBase");
dojo.require("dijit._TemplatedMixin");
dojo.require("dijit._WidgetsInTemplateMixin");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Form");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.importer.ImportClient");
dojo.require("rmps.importer.ImportDefinition");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.ui.editor.PageHelper");


dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("rmps.ui.pathmaps.PathmapsSection");
dojo.require("rmps.importer.ImportClient");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "PluginProfileImportEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.importer", "PluginProfileImportEditorMessages");

dojo.declare("rmps.importer.PluginProfileImportEditor", rmps.app.ui.internal.editors.BaseEditor, {
	"class": "rmps-plugin-profile-importer",
	
	//_definition: null,
	_overwrite: true,
	
	_requestButton: null,
	_form: null,
	_content: null,

	setForm: function(form) {
		this._form = form; 
	},
	
	createContentWidget: function() {
		//this._definition = new rmps.importer.ImportDefinition();
		this._loadCookie();
		return this._content = new rmps.importer.PluginProfileImportEditor._Content({_editor: this});
	},
	
	addButtons: function(toolbar) {
		this._requestButton = new rmps.ui.SimpleButton({
			label: messages.editorInputButton,
			onClick: dojo.hitch(this, this.requestImport)
		});
		toolbar.addWidget(this._requestButton);
	},
	
	_doWarning: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.makeVisible(warning);
	},
	
	_hideWarning: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.makeInvisible();
	},
	
	_disableRedo: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.disableRedo();
	},	
	
	requestImport: function() {
	    this._requestImport(this);
	},
	
	_requestImport: function(editor, duplicates) {
		if (!editor) {
			editor = this;
		}
	
		rmps.importer.ImportClient.requestPluginProfileImport(this._content.installData(), this._content.domainData()).then(dojo.hitch(this, function(result) {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: result,
				closable: true
			});
		}), dojo.hitch(this, function(error) {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.error,
				details: [{message: error.toString()}],
				closable: false
			});
		}), dojo.hitch(this, function(update) {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.importInProgress + update,
				closable: true
			});
		}));
		
	},
	
	_loadCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
			this._overwrite = !!cookie.overwrite;
		}
	},
	
	_saveCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		var cookie;
		if (value) {
			cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie.overwrite = this._overwrite;
		dojo.cookie(this.declaredClass, dojo.toJson(cookie), {expires: 90});
	},
	
	resize: function() {
		//override: do not resize
	}
});

dojo.declare("rmps.importer.PluginProfileImportEditor._Content", [dijit._WidgetBase, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], {
	templateString: dojo.cache("rmps.importer", "templates/PluginProfileImportEditor_Content.html"),
	_editor: null,
	messages: messages,
	
	postCreate: function() {
		this.inherited(arguments);
		this._editor.containerNode.appendChild(this.domNode);
	},

	_validateInput: function() {
		return dojo.every(["_domainName", "_domainId", "_domainUri", "_repoUri", "_featureId"], dojo.hitch(this, function (name) {
			return !!this[name].value;
		}));		
	},
	
	_onChange: function() {
		this._updateCb && this._updateCb(this._validateInput());
	},
	
	installData: function() {
		return {repositoryUri: this._repoUri.value, featureId: this._featureId.value};
	},
	
	domainData: function() {
		return {name: this._domainName.value, id: this._domainId.value, uri: this._domainUri.value, snapshot: this._snapshot.value};
	}
	
});

})();
