/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.ProfileImportEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Form");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.importer.ImportClient");
dojo.require("rmps.importer.ImportDefinition");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.ui.editor.PageHelper");


dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("rmps.ui.pathmaps.PathmapsSection");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "ProfileImportEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.importer", "ProfileImportEditorMessages");

var projectId;

var connectOnInput = function(obj, widget, method) {
	widget.connect(obj, "onchange", method);
};

dojo.declare("rmps.importer.ProfileImportEditor", rmps.app.ui.internal.editors.BaseEditor, {
	"class": "rmps-profile-importer",
	
	_definition: null,
	_overwrite: true,
	
	_requestButton: null,
	_form: null,

	setForm: function(form) {
		this._form = form; 
	},
	
	createContentWidget: function() {
		this._definition = new rmps.importer.ImportDefinition();
		this._loadCookie();
		return new rmps.importer.ProfileImportEditor._Content({_editor: this});
	},
	
	addButtons: function(toolbar) {
		this._requestButton = new rmps.ui.SimpleButton({
			label: messages.editorInputButton,
			onClick: dojo.hitch(this, this.requestImport)
		});
		toolbar.addWidget(this._requestButton);
	},
	
	_doWarning: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.makeVisible(warning);
	},
	
	_hideWarning: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.makeInvisible();
	},
	
	_disableRedo: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.disableRedo();
	},	
	
	requestImport: function() {
	    this._requestImport(this);
	},
	
	_requestImport: function(editor, duplicates) {
		if (!editor) {
			editor = this;
		}
	
		editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.editorPleaseWait,
			closable: false
		});
		
		var errors = [];
		this.contentWidget._pathmapsSection.validate(errors);
		
		var errorMessageStrings = [];
		//element 0 is always an array of all the pathmaps
		for (i = 1; i < errors.length; i++) {
			if (errors[i] instanceof Error) {
				errorMessageStrings.push({message: dojox.html.entities.encode(errors[i].message)});
			} else {
				errorMessageStrings.push({message: dojox.html.entities.encode(errors[i])});
			}
		}
		
		if (errorMessageStrings.length > 0) {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.editorImportPathmapError,
				details: errorMessageStrings,
				closable: true
			});
			return;
		}
		
			var deferred = rmps.importer.ImportClient.requestProfileImport(projectId, editor._overwrite, editor._definition.pathMaps, duplicates, editor._form.domNode);
			deferred.addCallback(editor, function(result) {
				
				if (result === "ok") {
					editor._hideWarning();
					this._refreshStatus();
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.OK,
						summary: dojo.string.substitute(
							messages.importRequestSuccessful,
							[rmps.core.util.IRmpsConstants.ADMIN_IMPORT_STATUS]),
						closable: true
					});
				} else if (result === "warning") {
					editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: messages.editorImportWarning,
						closable: true
					});
					editor._doWarning(result);
				} else {
					editor._hideWarning();
					if (result === "error") {
						if (result.missingMetamodel) {
							editor.showMessage({
								type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
								summary: dojo.string.substitute(messages.editorMissingDependentMetamodels, [result.missingMetamodel]),
								closable: true
							});
							return;
						}
					}
					
					editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.editorImportError,
						details: [ {message: messages['editorImportError.explanation'] + "<br>" + messages['editorImportError.useraction']}],
						closable: true
					});
				}
				
			});
			deferred.addErrback(editor, function(error) {
				editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.editorImportSeriousError,
					details: [ {message:dojo.string.substitute(messages['editorImportSeriousError.explanation'], [error.status]) + "<br>" + messages['editorImportSeriousError.useraction']}],
					closable: true
				});
			});

	},
	
	_refreshStatus: function() {
		var helper = rmps.ui.editor.PageHelper.getInstance();
		var statusEditor = helper.getEditor(
				new rmps.importer.ImportStatusEditorInput({projectId: projectId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	},

	_loadCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
			this._overwrite = !!cookie.overwrite;
		}
	},
	
	_saveCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		var cookie;
		if (value) {
			cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie.overwrite = this._overwrite;
		dojo.cookie(this.declaredClass, dojo.toJson(cookie), {expires: 90});
	},
	
	resize: function() {
		//override: do not resize
	}
});

dojo.declare("rmps.importer.ProfileImportEditor._Content", [dijit._Widget], {
	
	_editor: null,
	_mainWidget: null,
	_errorsAndWarningsWidget: null,
	_pathmapsSection: null,
	_definition: null,
	
	_init: function() {
		this._definition = this._editor._definition;
		
		this._errorsAndWarningsWidget = new rmps.importer.ProfileImportEditor._ErrorsAndWarningsContent({_editor :this._editor});
		this._editor.containerNode.appendChild(this._errorsAndWarningsWidget.domNode);
		this._errorsAndWarningsWidget.startup();

		this._mainWidget = new rmps.importer.ProfileImportEditor._SubContent({_editor :this._editor});
		this._editor.containerNode.appendChild(this._mainWidget.domNode);
		this._mainWidget.startup();
		
		this._pathmapsSection = new rmps.ui.pathmaps.PathmapsSection._PathMaps({editor: this});
		this._editor.containerNode.appendChild(this._pathmapsSection.domNode);
		this._pathmapsSection.startup();
		
		dojo.addClass(this._pathmapsSection.domNode, "container");
		dojo.removeClass(this._editor.containerNode, "disabled");
	},
	
	postCreate: function() {
		this.inherited(arguments);
		var groupId = null;
		projectId = jazz.app.context.get().itemId;
		
		this._init();
	}
	
});


dojo.declare("rmps.importer.ProfileImportEditor._SubContent", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	rowCount: 0,
	
	templateString: '<div class="container">' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" readOnly="false" uaKey="com.ibm.xtools.rmps.web.importer.page_MetamodelImportEditor001">' +
				 '<div class="metamodelImporterElement">' +
					'<span>${messages.editorTableMessage}</span>' +
				 '</div>' +
			'</div>' +
			'<div class="metamodelImporterElement">' +
				'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="vCb" dojoAttachEvent="onClick: _onValidate"/>' +
				'<span>${messages.editorOverwriteCheckbox}</span>' +
			'</div>' +
		'</div>',
				
	_editor: null,
	_help: null,
	
	postCreate: function() {
		this._children = [];
		
		this.vCb.attr("value", this._editor._overwrite);
		this._section.setPrimaryTitle(messages.editorBarTitle);
		
		var form = new dijit.form.Form({
			method: 'post',
			encType: 'multipart/form-data'
		});
		this._editor.setForm(form);
		this._section.containerNode.appendChild(form.domNode);
		
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.editorTableColumn1, className: "colOne"},
					{label: "", className: "actions"}
				]
		});
		form.domNode.appendChild(this._table.domNode);
		this._table.startup();
		
		this._section.addAction(messages.editorAddLink, messages.editorAddLinkTooltip, dojo.hitch(this, function() {
			this._addMetamodelRow();
			this._editor._disableRedo();
		}));		
	},
	
	_addMetamodelRow: function() {
		var row = new rmps.importer.ProfileImportEditor._WorkspaceRow({
			_editor: this._editor,
			rowCount: this.rowCount
		});
		this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		row.startup();
		
		this._children.push(row);
		var handle = this.connect(row, "uninitialize", function() {
			this.disconnect(handle);
			jazz.util.array.removeByReference(this._children, row);
		});

		this.rowCount++;
		row.focus();
	},	
	
	_onValidate: function() {
		this._editor._overwrite = !!this.vCb.attr("value");
		this._editor._saveCookie();
	},
	
	_toggleChoice: function(e) {
	},
	
	startup: function() {
		this.inherited(arguments);	
		
		this._section.startup();		
		
		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	}
	
});

dojo.declare("rmps.importer.ProfileImportEditor._ErrorsAndWarningsContent", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<div dojoAttachPoint="_results" class="container">' +
	'<div dojoAttachPoint="_warnings" class="container">' +
		'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="true" class="warningBox">' +
			'<table class="general warningsTable">' +
			'<tr>' +
			'<td dojoAttachPoint="_cell">' +
				'<div dojoAttachPoint="_notMetamodels"></div>' +
				'<div dojoAttachPoint="_duplicates"></div>' +
				'<div dojoAttachPoint="_existings"></div>' +
				'<div dojoAttachPoint="_invalidURIs"></div>' +
				'<div dojoAttachPoint="_duplicateFiles"></div>' +
			'</td>' +
			'</tr>' +
			'<tr>' +
			'<div dojoAttachPoint="_button"></div>' +
			'</tr>' +
			'</table>' +
		'</div>' +
		'</div>' +
    '</div>',
	
	_ignore: [],
	_editor: null,
	_help: null,
	_redoButton: null,
	
	postCreate: function() {
		this.makeInvisible();
		this._section.setPrimaryTitle(messages.editorWarningBarTitle);
		this.inherited(arguments);
	},
	
	disableRedo : function() {
		if (this._redoButton) {
			this._redoButton.setEnabled(false);
			this._redoButton.domNode.textContent = messages.editorReimportButtonDisabled;
		}
	},
	
	enableRedo : function() {
		if (this._redoButton) {
			this._redoButton.setEnabled(true);
			this._redoButton.domNode.textContent = messages.editorReimportButton;
		}
	},
	
	makeInvisible : function() {
		dojo.addClass(this._results, "hidden");
	},
	
	//example: showOneWarning(true/false, _notMetamodels, messages.editorImportNotMetamodels, notMetaModel, null or ignore
	showOneWarning : function(insertBreak, widget, warningMessage, warningArray, ignoreArray) {
		if (warningArray.length > 0) {
			dojo.removeClass(widget, "hidden");
			widget.innerHTML = ((insertBreak) ? "<p></p>": "") + warningMessage + "<ul>";
			for (var index=0; index < warningArray.length; index++){
				var value = warningArray[index];
				widget.innerHTML += "<li>" + value + "</li>";
				if (ignoreArray) {
					ignoreArray.push(value);
				}
			}
			widget.innerHTML += "</ul>";
		}
	},

	makeVisible : function(warning) {
		//only show the outer warning area
		dojo.removeClass(this._results, "hidden");
		dojo.removeClass(this._warnings, "hidden");
		dojo.addClass(this._notMetamodels, "hidden");
		dojo.addClass(this._duplicates, "hidden");
		dojo.addClass(this._duplicateFiles, "hidden");
		dojo.addClass(this._invalidURIs, "hidden");
		dojo.addClass(this._existings, "hidden");

		if (this._redoButton) {
			this.enableRedo();
			this._ignore.splice(0, this._ignore.length);
		} else {
			this._ignore = [];
			this._redoButton = new rmps.ui.SimpleButton({
				label: messages.editorReimportButton,
				onClick: dojo.hitch(this._editor, this._editor._requestImport, this._editor, this._ignore)
			}, this._button);
			
			this._redoButton.startup();
		}
		var notMetaModel = [], duplicate = [], existing = [], invalidURI = [], duplicateFilename = [];
		
		var foundWarning = false;
		for (var key in warning) {
		    var value = warning[key];
		    if (key.indexOf("notMetamodel") === 0) {
				notMetaModel.push(value);
			} else if(key.indexOf("duplicateFilename") === 0){
				duplicateFilename.push(value);
			} else if(key.indexOf("duplicate") === 0){
				duplicate.push(value);
			} else if(key.indexOf("existing") === 0){
				existing.push(value);
			} else if(key.indexOf("invalidURI") === 0){
				invalidURI.push(value);
			}
		}
		
		this.showOneWarning(false, this._notMetamodels, messages.editorImportNotMetamodels, notMetaModel, null);
		this.showOneWarning(notMetaModel.length > 0, this._duplicates, messages.editorImportDuplicated, duplicate, this._ignore);
		this.showOneWarning(notMetaModel.length > 0 || duplicate.length > 0, this._existings, messages.editorImportAlreadyExist, existing, this._ignore);
		this.showOneWarning(notMetaModel.length > 0 || duplicate.length > 0 || existing.length > 0, this._invalidURIs, messages.editorImportInvalidURIs, invalidURI, this._ignore);
		this.showOneWarning(notMetaModel.length > 0 || duplicate.length > 0 || existing.length > 0 || invalidURI.length > 0, this._duplicateFiles, messages.editorImportDuplicateFilename, duplicateFilename, this._ignore);
	},
	
	startup: function() {
		this.inherited(arguments);	
		
		var b = dojo.marginBox( this.domNode );
		
		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	}
	
});

dojo.declare("rmps.importer.ProfileImportEditor._WorkspaceRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_pathPreview"></span>' +
					//the width:100% does not work in Firefox
					'<input class="editable" type="file" name="thefile" style="width:100%;" size="80" dojoAttachPoint="_pathInput"></input>' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 26px;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	_editor: null,
	rowCount: 0,
	_filename: null,
	_definition: null,
	_originalWorkspace: null,
	
	postCreate: function() {
		connectOnInput(this._pathInput, this, function() {
			this._filename.path = this._pathInput.files;
			this._editor._disableRedo();
		});
		
		this._definition = this._editor._definition;
		this._children = [];
		
		this._filename = {
			path: ""
		};
		this._definition.workspaces.push(this._filename);
		
		this.toolbar.addButton(this, function(e) {
					dojo.stopEvent(e);
					this.remove();
					this.destroyRecursive();
				}, {
				label: messages.removeFile,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/remove.gif")
			});
		
		this._pathInput.name += this.rowCount;
	},
	
	focus: function() {
		this._pathInput.focus();
	},
	
	getChildren: function() {
		return this._children.slice();
	},
	
	remove: function() {
		dojo.forEach(this._children, function(row) {
			row.remove();
		});
		jazz.util.array.removeByReference(this._definition.workspaces, this._filename);
	},
	
	pruneEmpty: function() {
		dojo.forEach(this.getChildren(), function(row) { 
			if (!row.project.directoryName) {
				row.remove();
				row.destroyRecursive();
			}
		});
	}
});

})();
