/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("rsa.rmps.compare.RsaStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojo.i18n");
dojo.require("dijit.form.SimpleTextarea");

dojo.require('com.ibm.team.vvc.shared.web.ui.context.ContextUtils');
dojo.requireLocalization("rsa.rmps.compare", "CompareWizardApplicationMessages");

(function() {
var messages;
var WizardStep = jazz.ui.wizard.WizardStep;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

dojo.declare("rsa.rmps.compare.RsaStep", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl('rsa.rmps', 'compare/templates/RsaStep.html'),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("rsa.rmps.compare", "CompareWizardApplicationMessages");
	},

	postCreate: function() {
		var uri = ContextUtils.getRequestParmValue("interimResultUri");
		if (uri) {
			uri = decodeURIComponent(uri);
			var args = {
					url: net.jazz.ajax.contextRoot() + '/vvcshared/proxy'+
						'?right_now='+new Date().getTime(), //$NON-NLS-1$   
					headers: {
						"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json",
						"vvc_servicePath" : uri
					},
					sync: true,
					handleAs: "json",
					load: dojo.hitch(this, this.loadCompareDate),
					error: dojo.hitch(this, this.onError),
					failOk: true
			};
			
			jazz.client.xhrGet(args);
		}
	},
	
	loadCompareDate: function(response, ioArgs) {
		if (response) {
			var resourceMap = {}; 
			var hasResources = hasResources || this.extractResources(response.ancestor, resourceMap);
			var hasResources = hasResources || this.extractResources(response.descendant, resourceMap);
			var hasResources = hasResources || this.extractResources(response.otherDescendant, resourceMap);
			
			if (hasResources) {
				this.ui_listSection.style.visibility = "visible";
				var textStr = "";
				for (var resourceUri in resourceMap) {
					textStr += resourceMap[resourceUri] + "\n";
				}
				this.ui_listText.set("value", textStr);
			}
		}
	},
	
	extractResources: function(contributor, resourceMap) {
		var retVal = false;
		if (contributor && contributor.resourceDiffs && contributor.resourceDiffs.length > 0) {
			var resourceDiffs = contributor.resourceDiffs;
			for (var i = 0; i < resourceDiffs.length; i++) {
				var resourceDiff = resourceDiffs[i];
				if (resourceDiff.deltaType && resourceDiff.version && resourceDiff.displayName && 
						resourceDiff.compareProvider && 
						resourceDiff.compareProvider == "http://jazz.net/dm/rsa/compare") {
					resourceMap[resourceDiff.version] = resourceDiff.displayName;
					retVal = true;
				}
			}
		}
		
		return retVal;
	},
	
	onError: function(response, ioArgs) {
		alert(this.messages.interimResultsUnavailable);
		
		console.log("Failed to load resource '"
				+ ioArgs.args.headers.vvc_servicePath
				+ "'...Invalid Response Status("
				+ ioArgs.xhr.status + "): "
				+ response.message);
	},
	
	canFinish: function() {
		return true;
	},

});

})();